@@ -45,8 +45,8 @@ class Node:
 
 class CreateJob:
     #  inputs : List[Node] = None, outputs : List[Node] = None
-    def __init__(self, uid: str, name: str, version: str, description: str = None, inputs=None, outputs=None,
-                 meta: JobMetadata = None, *args, **context):
+    def __init__(self, uid: str, name: str, description: str = None, inputs=None, outputs=None,
+                 meta: JobMetadata = None, span_uid=None, bounded_by_span=False, pipeline_run_id=None, *args, **context):
         """
         Description:
             create job class used to create job in torch catalog
@@ -56,8 +56,10 @@ class CreateJob:
         :param inputs: (list[Node]) input for the job
         :param outputs: (list[Node]) output for the job
         :param meta: job metadata
-        :param pipeline: pipeline object for which you want to add a job
         :param context: context
+        :param pipeline_run_id: id of pipeline_run
+        :param bounded_by_span: True if the job has to be bounded with a span
+        :param span_uid: uid of new span to be created
         """
         if outputs is None:
             outputs = []
@@ -65,13 +67,15 @@ class CreateJob:
             inputs = []
         self.uid = uid
         self.name = name
-        self.version = version
         self.description = description
         if meta is not None:
             self.meta = JobMetadata(meta.owner, meta.team, meta.codeLocation)
         self.inputs = inputs
         self.outputs = outputs
         self.context = context
+        self.span_uid = span_uid
+        self.bounded_by_span = bounded_by_span
+        self.pipeline_run_id = pipeline_run_id
 
     def __eq__(self, other):
         return self.uid == other.uid
