@@ -21,9 +21,11 @@ from requests_toolbelt import MultipartEncoder
 from acceldata_sdk.models.dqrule import DataQualityExecutionResult, DataQualityRule, DataQualityRuleResource
 from acceldata_sdk.models.reconcillationrule import ReconcillationExecutionResult, ReconciliationRuleResource
 from acceldata_sdk.models.ruleExecutionResult import RuleResult, PolicyFilter, ExecutionPeriod, RuleType, RuleExecutionSummary
-from acceldata_sdk.constants import AssetSourceType, SdkSupportedVersions
+from acceldata_sdk.constants import AssetSourceType, SdkSupportedVersions, TorchBuildVersion
 from acceldata_sdk.common import Executor
 from acceldata_sdk.constants import PolicyType, FailureStrategy, RuleExecutionStatus
+from semantic_version import Version, SimpleSpec
+
 
 from typing import List
 from dataclasses import asdict
@@ -33,6 +35,7 @@ from datetime import datetime, timedelta
 _HEADERS = {'User-Agent': 'Acceldata-sdk', 'accessKey': None, 'secretKey': None, 'Content-Type': 'application/json'}
 catalog_api_path = "/catalog-server/api"
 pipeline_api_path = "/torch-pipeline/api"
+admin_api_path = "/admin/api"
 
 import urllib3
 
@@ -70,6 +73,13 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         self._timeout = 10000
         self._catalog_api_base = f"{url}{catalog_api_path}"
         self._pipeline_api_base = f"{url}{pipeline_api_path}"
+        is_onprem = url.endswith('/torch')
+        loc = url.rfind('/torch')
+        if is_onprem:
+            self._admin_api_base = f"{url[:loc]}{admin_api_path}"
+        else:
+            self._admin_api_base = f"{url}{admin_api_path}"
+
         if access_key is None and secret_key is None:
             raise Exception('Access key and secret key - required')
         self.access_key = access_key
@@ -97,6 +107,12 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         )
         return SdkSupportedVersions(**response['versions'])
 
+    def get_torch_version(self) -> TorchBuildVersion:
+        response = self._get(
+            f'{self._admin_api_base}/build-details'
+        )
+        return TorchBuildVersion(**response)
+
     # function to create pipeline
     def create_pipeline(self, pipeline: CreatePipeline) -> Pipeline:
         """
@@ -248,17 +264,47 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return PipelineRun(**response['run'])
 
     # get pipeline run with particular continuation id or run id
-    def get_pipeline_run(self, pipeline_run_id: int) -> PipelineRun:
+    def get_pipeline_run(self, pipeline_run_id: int = None, continuation_id: str = None, pipeline_id: str = None) -> PipelineRun:
         """
             Description:
-                To get pipeline run with particular id of any pipeline
-        :param pipeline_run_id: continuation id or run id of the pipeline run
+                To get pipeline run with particular id or continuation id of any pipeline
+        :param pipeline_run_id: run id of the pipeline run
+        :param continuation_id: continuation id of the pipeline run
+        :param pipeline_id: id of the pipeline. This is a mandatory parameter when run is being queried using continuation_id
         :return: PipelineRun instance
         """
-        url = f'{self._pipeline_api_base}/pipelines/runs/{pipeline_run_id}'
-        response = self._get(url)
-        response['run']['client'] = self
-        return PipelineRun(**response['run'])
+        payload = dict()
+        return_run = dict()
+        if pipeline_run_id is not None:
+            url = f'{self._pipeline_api_base}/pipelines/runs/{pipeline_run_id}'
+            response = self._get(url)
+            return_run = response['run']
+        elif continuation_id is not None and pipeline_id is not None:
+            supported_versions = self.get_supported_sdk_versions()
+            torch_version: str = self.get_torch_version()
+            if torch_version.buildVersion is not None and torch_version.buildVersion != "":
+                actual_version = torch_version.buildVersion[:5]
+            else:
+                actual_version = supported_versions.maxVersion
+            ver_comparator = SimpleSpec(f'<={actual_version}')
+            if Version('2.5.0') in ver_comparator:
+                url = f'{self._pipeline_api_base}/pipelines/{pipeline_id}/runs'
+                payload = {"continuationId": continuation_id}
+                response = self._get(url, params=payload)
+                return_run = response["runs"][0]
+            else:
+                # For torch versions lower than 2.5.0 use old call for continuation_id
+                if continuation_id.isdigit() is False:
+                    url = f'{self._pipeline_api_base}/pipelines/runs/{continuation_id}'
+                    response = self._get(url)
+                    return_run = response['run']
+                else:
+                    raise TorchSdkException('Please provide continuation_id which is not a valid number.')
+        else:
+            raise TorchSdkException('Please provide either continuation_id and pipeline_id or pipeline_run_id')
+
+        return_run['client'] = self
+        return PipelineRun(**return_run)
 
     # get pipeline runs for a pipeline
     def get_pipeline_runs(self, pipeline_id: int) -> List[PipelineRun]:
@@ -283,7 +329,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         Description:
             used to create span for any pipeline run
         :param pipeline_run_id:
-        :param span: Span class instace
+        :param span: Span class instance
         :return: Span
         """
         response = self._post(
@@ -329,9 +375,14 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         response = self._get(url, params=params)
         print(f'response: {response}')
         return_value = dict()
-        return_value['span'] = response['spans'][0]
+        return_value['span'] = dict()
         return_value['span']['client'] = self
-        return Span(**return_value['span'])
+        if response['spans']:
+            return_value['span'] = response['spans'][0]
+            return_value['span']['client'] = self
+            return Span(**return_value['span'])
+        else:
+            return None
 
     def get_spans(self, pipeline_run_id):
         response = self._get(
@@ -720,7 +771,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return RuleCancelResponse(**response['result'])
 
     def execute_rule(self, rule_type: PolicyType, rule_id, sync=True, incremental=False,
-                     failure_strategy: FailureStrategy = FailureStrategy.DoNotFail):
+                     failure_strategy: FailureStrategy = FailureStrategy.DoNotFail, pipeline_run_id=None):
         """
         Description:
             To execute rule synchronously and asynchronously
@@ -732,7 +783,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
             fail at warning or never fail
         """
         rule_executor = Executor(rule_type, self, sync=sync)
-        rule_executor.execute(rule_id, incremental, failure_strategy=failure_strategy)
+        rule_executor.execute(rule_id, incremental, failure_strategy=failure_strategy, pipeline_run_id=pipeline_run_id)
         return rule_executor
 
     def get_rule_status(self, rule_type, execution_id) -> RuleExecutionStatus:
@@ -757,20 +808,26 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         rule_executor = Executor(rule_type, self)
         return rule_executor.get_execution_result(execution_id, failure_strategy=failure_strategy)
 
-    def execute_dq_rule(self, rule_id, incremental=False) -> RuleExecution:
-        payload_temp = {"executionType": "FULL"}
+    def execute_dq_rule(self, rule_id, incremental=False, pipeline_run_id=None) -> RuleExecution:
+        payload_temp = dict()
+        payload_temp['executionType'] = "FULL"
         if incremental:
-            payload_temp = {"executionType": "INCREMENTAL"}
+            payload_temp['executionType'] = "INCREMENTAL"
+        if pipeline_run_id is not None:
+            payload_temp['pipelineRunId'] = pipeline_run_id
         response = self._post(
             url=f'{self._catalog_api_base}/rules/data-quality/{rule_id}/executions',
             payload=payload_temp
             )
         return RuleExecution(**response['execution'])
 
-    def execute_reconciliation_rule(self, rule_id, incremental=False) -> RuleExecution:
-        payload_temp = {"executionType": "FULL"}
+    def execute_reconciliation_rule(self, rule_id, incremental=False, pipeline_run_id=None) -> RuleExecution:
+        payload_temp = dict()
+        payload_temp['executionType'] = "FULL"
         if incremental:
-            payload_temp = {"executionType": "INCREMENTAL"}
+            payload_temp['executionType'] = "INCREMENTAL"
+        if pipeline_run_id is not None:
+            payload_temp['pipelineRunId'] = pipeline_run_id
         response = self._post(
             url=f'{self._catalog_api_base}/rules/reconciliation/{rule_id}/executions',
             payload=payload_temp
@@ -824,7 +881,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         rule_id = identifier
         if not check_int(identifier):
             if rule_type.value == RuleType.DATA_QUALITY.value:
-                rule_res =  self.get_dq_rule(identifier)
+                rule_res = self.get_dq_rule(identifier)
             elif rule_type.value == RuleType.RECONCILIATION.value:
                 rule_res = self.get_reconciliation_rule(identifier)
             rule_id = rule_res.rule.id
